import { AccessPermissionsContract } from '@nestjs-bff/global-contracts/lib/domain/access-permissions/access-permissions.contract';
import { IEntity } from '@nestjs-bff/global-contracts/lib/domain/core/entity.interface';
import { Document, Model } from 'mongoose';
import { AuthCheckContract } from '../../../shared/authchecks/authcheck.contract';
import { CrudOperations } from '../../../shared/authchecks/crud-operations.enum';
import { CacheStore } from '../../../shared/caching/cache-store.shared';
import { LoggerSharedService } from '../../../shared/logging/logger.shared.service';
import { ClassValidator } from '../validators/class-validator';
export interface IBaseRepoParams<TEntity extends IEntity> {
    loggerService: LoggerSharedService;
    model: Model<Document & TEntity>;
    cacheStore: CacheStore;
    defaultTTL: number;
    entityValidator: ClassValidator<TEntity>;
    entityAuthChecker?: AuthCheckContract<IEntity, CrudOperations>;
}
/**
 * Base repo query repository
 *
 * Notes:
 *  - By default will try to validate that org and user filtering in in place, unless overridden with params
 *  - FindAll can be achieved with find, passing no conditions
 */
export declare abstract class BaseRepo<TEntity extends IEntity> {
    private readonly name;
    protected readonly loggerService: LoggerSharedService;
    protected readonly model: Model<Document & TEntity>;
    protected readonly cacheStore: CacheStore;
    protected readonly defaultTTL: number;
    readonly modelName: string;
    readonly entityValidator: ClassValidator<TEntity>;
    readonly entityAuthChecker: AuthCheckContract<IEntity, CrudOperations>;
    constructor(params: IBaseRepoParams<TEntity>);
    findById(id: string, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        skipCache?: boolean;
        ttl?: number;
    }): Promise<TEntity>;
    tryFindById(id: string, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        skipCache?: boolean;
        ttl?: number;
    }): Promise<TEntity | null>;
    findOne(conditions: object, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        skipCache?: boolean;
        ttl?: number;
    }): Promise<TEntity>;
    tryFindOne(conditions: object, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        skipCache?: boolean;
        ttl?: number;
    }): Promise<TEntity | null>;
    find(conditions: object, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        skipCache?: boolean;
        ttl?: number;
    }): Promise<TEntity[]>;
    create(newEntity: Partial<TEntity>, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        customValidator?: ClassValidator<TEntity>;
    }): Promise<TEntity>;
    patch(patchEntity: Partial<TEntity>, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        customValidator?: ClassValidator<TEntity>;
    }): Promise<TEntity>;
    update(entity: TEntity, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
        customValidator?: ClassValidator<TEntity>;
    }): Promise<TEntity>;
    delete(id: string, options?: {
        accessPermissions?: AccessPermissionsContract;
        skipAuthorization?: boolean;
    }): Promise<TEntity | undefined>;
    protected clearCacheByEntity(entity: TEntity, options?: {
        customValidator?: ClassValidator<TEntity>;
    }): Promise<void>;
    protected clearCacheByKey(cacheKey: string): Promise<any>;
    protected abstract generateValidQueryConditionsForCacheClear(entity: TEntity): object[];
    protected _dbFindOne(conditions: object): Promise<(Document & TEntity) | null>;
    protected _dbFind(conditions: object): Promise<Array<Document & TEntity>>;
    protected _dbSave(createModel: Document & TEntity): Promise<Document & TEntity>;
    protected _dbRemove(deleteModel: Document & TEntity): Promise<Document & TEntity>;
    protected _dbFindById(id: any): Promise<Document & TEntity | null>;
    protected _dbFindOneAndReplace(entity: Partial<TEntity>): Promise<any>;
}
//# sourceMappingURL=base.repo.d.ts.map