"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const crud_operations_enum_1 = require("../../../shared/authchecks/crud-operations.enum");
const scoped_entity_authcheck_1 = require("../../../shared/authchecks/scoped-entity.authcheck");
const caching_utils_1 = require("../../../shared/caching/caching.utils");
const app_exception_1 = require("../../../shared/exceptions/app.exception");
/**
 * Base repo query repository
 *
 * Notes:
 *  - By default will try to validate that org and user filtering in in place, unless overridden with params
 *  - FindAll can be achieved with find, passing no conditions
 */
class BaseRepo {
    constructor(params) {
        this.loggerService = params.loggerService;
        this.model = params.model;
        this.name = `RepoBase<${this.model.modelName}>`;
        this.modelName = this.model.modelName;
        this.cacheStore = params.cacheStore;
        this.defaultTTL = params.defaultTTL;
        this.entityValidator = params.entityValidator;
        this.entityAuthChecker = params.entityAuthChecker || new scoped_entity_authcheck_1.ScopedEntityAuthCheck();
    }
    //
    // findOne
    //
    findById(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.tryFindById(id, options);
            // validate not null
            if (!result)
                throw new app_exception_1.AppError(`Could not find entity ${this.name} with id ${id}`, options);
            // Return
            return result;
        });
    }
    //
    // tryFindOne
    //
    tryFindById(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.findOneById`, id, options);
            // Setup
            let key;
            let result;
            let cachedResult;
            options = options || {}; // ensure options is not null
            // cache access
            if (!options.skipCache === true) {
                key = caching_utils_1.CachingUtils.makeCacheKeyFromId(id);
                cachedResult = yield this.cacheStore.get(key);
            }
            if (cachedResult) {
                result = cachedResult;
            }
            else {
                // data store access
                result = yield this._dbFindById(id);
            }
            // cache population
            if (!options.skipCache === true && result && !cachedResult) {
                // tslint:disable-next-line:no-non-null-assertion
                this.cacheStore.set(key, result, { ttl: options.ttl || this.defaultTTL });
            }
            // authorization checks
            if (!options.skipAuthorization && result) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: result,
                    operation: crud_operations_enum_1.CrudOperations.read,
                });
            }
            // Return
            return result;
        });
    }
    //
    // findOne
    //
    findOne(conditions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.tryFindOne(conditions, options);
            // validate not null
            if (!result)
                throw new app_exception_1.AppError(`Could not find entity ${this.name} with conditions ${JSON.stringify(conditions)}`, options);
            // Return
            return result;
        });
    }
    //
    // tryFindOne
    //
    tryFindOne(conditions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.findOne`, conditions, options);
            // Setup
            let key;
            let result;
            let cachedResult;
            options = options || {}; // ensure options is not null
            // cache access
            if (!options.skipCache === true) {
                key = caching_utils_1.CachingUtils.makeCacheKeyFromObject(conditions);
                cachedResult = yield this.cacheStore.get(key);
            }
            if (cachedResult) {
                result = cachedResult;
            }
            else {
                // data store access
                result = yield this._dbFindOne(conditions);
            }
            // cache population
            if (!options.skipCache === true && result && !cachedResult) {
                // tslint:disable-next-line:no-non-null-assertion
                this.cacheStore.set(key, result, { ttl: options.ttl || this.defaultTTL });
            }
            // authorization checks
            if (!options.skipAuthorization && result) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: result,
                    operation: crud_operations_enum_1.CrudOperations.read,
                });
            }
            // Return
            return result;
        });
    }
    //
    // find
    //
    find(conditions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.find`, conditions, options);
            // setup
            let key;
            let result;
            let cachedResult;
            options = options || {}; // ensure options is not null
            // cache access
            if (!options.skipCache === true) {
                key = caching_utils_1.CachingUtils.makeCacheKeyFromObject(conditions);
                cachedResult = yield this.cacheStore.get(key);
            }
            if (cachedResult) {
                result = cachedResult;
            }
            else {
                // data store access
                result = yield this._dbFind(conditions);
            }
            // cache population
            if (!options.skipCache === true && result && !cachedResult) {
                // tslint:disable-next-line:no-non-null-assertion
                this.cacheStore.set(key, result, { ttl: options.ttl || this.defaultTTL });
            }
            // authorization checks
            if (!options.skipAuthorization && result) {
                for (const entity of result) {
                    if (!options.skipAuthorization) {
                        yield this.entityAuthChecker.ensureAuthorized({
                            accessPermissions: options.accessPermissions,
                            origin: this.name,
                            targetResource: entity,
                            operation: crud_operations_enum_1.CrudOperations.read,
                        });
                    }
                }
            }
            // return
            return result;
        });
    }
    //
    // create
    //
    create(newEntity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.create`, newEntity, options);
            // setup
            options = options || {}; // ensure options is not null
            const validator = options.customValidator || this.entityValidator;
            // validation
            validator.validate(newEntity);
            // authorization checks
            if (!options.skipAuthorization) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: newEntity,
                    operation: crud_operations_enum_1.CrudOperations.create,
                });
            }
            // transfer values to the model
            const createModel = new this.model();
            Object.assign(createModel, newEntity);
            // persist
            return this._dbSave(createModel);
        });
    }
    //
    // patch
    //
    patch(patchEntity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.patch`, patchEntity, options);
            // setup
            options = options || {}; // ensure options is not null
            const validator = options.customValidator || this.entityValidator;
            // partial validation
            validator.validate(patchEntity);
            // fetch entity
            let fullModel = yield this._dbFindById(patchEntity.id);
            if (!fullModel)
                throw new app_exception_1.AppError(`No ${this.modelName} found with id ${patchEntity.id}`);
            // merge values
            fullModel = _.merge(fullModel, patchEntity);
            // full validation
            yield this.entityValidator.validate(fullModel);
            // authorization checks
            if (!options.skipAuthorization) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: fullModel,
                    operation: crud_operations_enum_1.CrudOperations.update,
                });
            }
            // persist
            const savedFullModel = yield this._dbSave(fullModel);
            // clear cache
            this.clearCacheByEntity(fullModel);
            return savedFullModel;
        });
    }
    //
    // update
    //
    update(entity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.update`, entity, options);
            // setup
            options = options || {}; // ensure options is not null
            const validator = options.customValidator || this.entityValidator;
            // validation
            yield validator.validate(entity, { skipMissingProperties: false });
            // authorization checks
            if (!options.skipAuthorization) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: entity,
                    operation: crud_operations_enum_1.CrudOperations.update,
                });
            }
            // persist
            const savedReplacedModel = yield this._dbFindOneAndReplace(entity);
            // clear cache
            this.clearCacheByEntity(savedReplacedModel);
            return savedReplacedModel;
        });
    }
    //
    // delete
    //
    delete(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug logging
            this.loggerService.debug(`${this.name}.delete`, id, options);
            // setup
            let deletedEntity;
            options = options || {}; // ensure options is not null
            // retrieve
            const deleteModel = yield this._dbFindById(id);
            if (!deleteModel)
                throw new app_exception_1.AppError(`No ${this.modelName} found with id ${id}`);
            // authorization checks
            if (!options.skipAuthorization && deleteModel) {
                yield this.entityAuthChecker.ensureAuthorized({
                    accessPermissions: options.accessPermissions,
                    origin: this.name,
                    targetResource: deleteModel,
                    operation: crud_operations_enum_1.CrudOperations.delete,
                });
            }
            if (deleteModel) {
                // persist deletion
                deletedEntity = yield this._dbRemove(deleteModel);
                // clear cache
                this.clearCacheByEntity(deletedEntity);
            }
            return deletedEntity;
        });
    }
    //
    // utility methods
    //
    clearCacheByEntity(entity, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // setup
            options = options || {}; // ensure options is not null
            const validator = options.customValidator || this.entityValidator;
            // validation
            yield validator.validate(entity);
            // clear by ID
            this.clearCacheByKey(caching_utils_1.CachingUtils.makeCacheKeyFromId(entity.id));
            // clear by query conditions
            this.generateValidQueryConditionsForCacheClear(entity).forEach(cacheClearEntity => {
                this.clearCacheByKey(caching_utils_1.CachingUtils.makeCacheKeyFromObject(cacheClearEntity));
            });
        });
    }
    clearCacheByKey(cacheKey) {
        if (cacheKey.trim.length > 0)
            throw new app_exception_1.AppError('cacheKey can not be null or whitespace');
        return this.cacheStore.del(cacheKey);
    }
    //
    // Abstracted Mongoose calls, to allow for easier testing through mocked mongoose calls
    //
    _dbFindOne(conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbFindOne`, conditions);
            const result = yield this.model.findOne(conditions).exec();
            return result;
        });
    }
    _dbFind(conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbFind`, conditions);
            return this.model.find(conditions).exec();
        });
    }
    _dbSave(createModel) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbSave`, createModel);
            return createModel.save();
        });
    }
    _dbRemove(deleteModel) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbRemove`, deleteModel);
            return deleteModel.remove();
        });
    }
    _dbFindById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbFindById`, id);
            return this.model.findById(id).exec();
        });
    }
    _dbFindOneAndReplace(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loggerService.debug(`${this.name}._dbFindOneAndReplace`, entity);
            const result = yield this.model.collection.findOneAndReplace({ id: entity.id }, entity, { returnOriginal: false });
            return result.value;
        });
    }
}
exports.BaseRepo = BaseRepo;
//# sourceMappingURL=base.repo.js.map