"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const organization_repo_1 = require("../../../../domain/organization/repo/organization.repo");
const app_shared_constants_1 = require("../../../../shared/app/app.shared.constants");
const caching_shared_constants_1 = require("../../../../shared/caching/caching.shared.constants");
const logger_shared_service_1 = require("../../../../shared/logging/logger.shared.service");
const server_exception_1 = require("../exceptions/server.exception");
const core_utils_1 = require("../utils/core.utils");
/**
 *
 *
 * @summary
 *  Checks authorization against a configured authcheck
 *  Defaults false: no authcheck found means no authorization
 *  Looks for authcheck from handler - first on handler method, then on controller
 *
 * @export
 * @class ResourceGuard
 * @implements {CanActivate}
 */
let AuthorizationGuard = class AuthorizationGuard {
    constructor(reflector, organizationRepo, cacheStore, nestjsBffConfig, logger) {
        this.reflector = reflector;
        this.organizationRepo = organizationRepo;
        this.cacheStore = cacheStore;
        this.nestjsBffConfig = nestjsBffConfig;
        this.logger = logger;
    }
    canActivate(context) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const req = context.switchToHttp().getRequest();
                if (!req)
                    throw new server_exception_1.BadGatewayHttpError('request can not be null');
                this.logger.trace('AuthorizationGuard.canActivate', {
                    'req.originalUrl': req.originalUrl,
                });
                // test to see if public route
                if (this.nestjsBffConfig.http.publicRouteRegex.test(req.originalUrl)) {
                    this.logger.debug('Public Route', req.originalUrl);
                    return true;
                }
                // get authorization from request
                const accessPermissions = req.accessPermissions;
                if (!accessPermissions) {
                    this.logger.warn('Route not public, but no authorization found. ', {
                        request: core_utils_1.getReqMetadataLite(req),
                    });
                    return false;
                }
                // debug logging
                this.logger.debug('AuthorizationGuard', {
                    'req.params': req.params,
                    'req.route': req.route,
                    accessPermissions,
                });
                // get key date for auth tests
                const orgId = yield this.getOrgIdFromSlug(req.params['organizationSlug']);
                const userId = req.params['userId'];
                const authchecks = yield this.getauthchecksFromCache(context);
                // Default false. No authcheck configured, not authorization
                if (!authchecks || authchecks.length === 0) {
                    this.logger.warn('Request not authorized', {
                        request: core_utils_1.getReqMetadataLite(req),
                        authorization: accessPermissions,
                    });
                    return false;
                }
                // run tests
                for (const authcheck of authchecks) {
                    const authCheckData = {
                        origin: 'AuthorizationGuard',
                        accessPermissions,
                        targetResource: {
                            orgId,
                            userId,
                        },
                    };
                    if (!(yield authcheck.isAuthorized(authCheckData))) {
                        this.logger.warn(`authcheck failed for ${authcheck.constructor.name}: ${JSON.stringify({ authcheck, authCheckData }, null, 2)}`);
                        return false;
                    }
                }
                // if we made it here we passed all the tests.  return true
                // this.logger.debug(`authcheck passed`, {
                //   authorization: accessPermissions,
                //   orgId,
                // });
                return true;
            }
            catch (error) {
                this.logger.error('Error in AuthorizationGuard', error);
                return false;
            }
        });
    }
    /**
     *
     * @param organizationSlug
     */
    getOrgIdFromSlug(organizationSlug) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!organizationSlug) {
                this.logger.debug('organizationSlug not found');
                return undefined;
            }
            // this.logger.debug('organizationSlug found', organizationSlug);
            const organization = yield this.organizationRepo.findOne({ slug: organizationSlug }, { skipAuthorization: true });
            if (!organization) {
                this.logger.debug('orgId not found for slug', organizationSlug);
                return undefined;
            }
            this.logger.debug('orgId found for slug', {
                organizationSlug,
                orgId: organization.id,
            });
            return organization.id;
        });
    }
    getauthchecksFromCache(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const authcheckCacheKey = `AuthorizationGuard-authcheck?class=${context.getClass()}&handler=${context.getHandler()})`;
            return this.cacheStore.wrap(authcheckCacheKey, () => this.getauthcheck(context), 
            // This configuration does not change dynamically.  Cache for a week
            { ttl: 60 * 60 * 24 * 7 });
        });
    }
    getauthcheck(context) {
        return __awaiter(this, void 0, void 0, function* () {
            let authchecks = this.reflector.get('authorization', context.getHandler());
            if (!authchecks) {
                authchecks = this.reflector.get('authorization', context.getClass());
            }
            return authchecks;
        });
    }
};
AuthorizationGuard = __decorate([
    common_1.Injectable(),
    __param(2, common_1.Inject(caching_shared_constants_1.CachingProviderTokens.Services.CacheStore)),
    __param(3, common_1.Inject(app_shared_constants_1.AppSharedProviderTokens.Config.App)),
    __metadata("design:paramtypes", [core_1.Reflector,
        organization_repo_1.OrganizationRepo, Object, Object, logger_shared_service_1.LoggerSharedService])
], AuthorizationGuard);
exports.AuthorizationGuard = AuthorizationGuard;
//# sourceMappingURL=authorization.guard.js.map